@@ -12,7 +12,7 @@ from pywink.api import get_light_bulbs, get_garage_doors, get_locks, \
     get_powerstrips, get_shades, get_sirens, \
     get_switches, get_thermostats, get_fans, get_air_conditioners, \
     get_propane_tanks, get_robots, get_scenes, get_light_groups, \
-    get_binary_switch_groups
+    get_binary_switch_groups, get_water_heaters
 
 from pywink.api import get_all_devices, get_eggtrays, get_sensors, \
     get_keys, get_piggy_banks, get_smoke_and_co_detectors, \
