@@ -3,10 +3,11 @@ import time
 import logging
 import urllib.parse
 
+import requests
+
 from pywink.devices import types as device_types
 from pywink.devices.factory import build_device
 
-import requests
 try:
     import urllib3
     from urllib3.exceptions import InsecureRequestWarning
@@ -78,7 +79,7 @@ class WinkApiInterface(object):
             else:
                 raise WinkAPIException("Failed to refresh access token.")
         response_json = arequest.json()
-        _LOGGER.debug(response_json)
+        _LOGGER.debug('%s', response_json)
         return response_json
 
     # pylint: disable=bare-except
@@ -116,11 +117,11 @@ class WinkApiInterface(object):
                                         data=json.dumps(state),
                                         headers=LOCAL_API_HEADERS,
                                         verify=False, timeout=3)
-            except:
+            except requests.exceptions.RequestException:
                 _LOGGER.error("Error sending local control request. Sending request online")
                 return self.set_device_state(device, state, id_override, type_override)
             response_json = arequest.json()
-            _LOGGER.debug(response_json)
+            _LOGGER.debug('%s', response_json)
             temp_state = device.json_state
             for key, value in response_json["data"]["last_reading"].items():
                 temp_state["last_reading"][key] = value
@@ -149,17 +150,16 @@ class WinkApiInterface(object):
                                         object_type, object_id)
         arequest = requests.get(url_string, headers=API_HEADERS)
         response_json = arequest.json()
-        _LOGGER.debug(response_json)
+        _LOGGER.debug('%s', response_json)
         return response_json
 
-    # pylint: disable=bare-except
+    # pylint: disable=bare-except, too-many-locals
     def local_get_state(self, device, id_override=None, type_override=None):
         """
         Get device state via local API, and fall back to online API.
 
         Args:
             device (WinkDevice): The device the change is being requested for.
-            state (Dict): The state being requested.
             id_override (String, optional): A device ID used to override the
                 passed in device's ID. Used to make changes on sub-devices.
                 i.e. Outlet in a Powerstrip. The Parent device's ID.
@@ -189,11 +189,11 @@ class WinkApiInterface(object):
                 arequest = requests.get(url_string,
                                         headers=LOCAL_API_HEADERS,
                                         verify=False, timeout=3)
-            except:
+            except requests.exceptions.RequestException:
                 _LOGGER.error("Error sending local control request. Sending request online")
                 return self.get_device_state(device, id_override, type_override)
             response_json = arequest.json()
-            _LOGGER.debug(response_json)
+            _LOGGER.debug('%s', response_json)
             temp_state = device.json_state
             for key, value in response_json["data"]["last_reading"].items():
                 temp_state["last_reading"][key] = value
@@ -221,10 +221,13 @@ class WinkApiInterface(object):
         url_string = "{}/{}s/{}/update_firmware".format(self.BASE_URL,
                                                         object_type,
                                                         object_id)
-        arequest = requests.post(url_string,
-                                 headers=API_HEADERS)
-        response_json = arequest.json()
-        return response_json
+        try:
+            arequest = requests.post(url_string,
+                                     headers=API_HEADERS)
+            response_json = arequest.json()
+            return response_json
+        except requests.exceptions.RequestException:
+            return None
 
     def remove_device(self, device, id_override=None, type_override=None):
         """
@@ -238,19 +241,53 @@ class WinkApiInterface(object):
             type_override (String, optional): Used to override the device type
                 when a device inherits from a device other than WinkDevice.
         Returns:
-            response_json (boolean): True if the device was removed.
+            (boolean): True if the device was removed.
         """
         object_id = id_override or device.object_id()
         object_type = type_override or device.object_type()
         url_string = "{}/{}s/{}".format(self.BASE_URL,
                                         object_type,
                                         object_id)
-        arequest = requests.delete(url_string,
-                                   headers=API_HEADERS)
-        if arequest.status_code == 204:
-            return True
-        _LOGGER.error("Failed to remove device. Status code: " + arequest.status_code)
-        return False
+
+        try:
+            arequest = requests.delete(url_string,
+                                       headers=API_HEADERS)
+            if arequest.status_code == 204:
+                return True
+            _LOGGER.error("Failed to remove device. Status code: %s", arequest.status_code)
+            return False
+        except requests.exceptions.RequestException:
+            _LOGGER.error("Failed to remove device.")
+            return False
+
+    def create_lock_key(self, device, new_device_json, id_override=None, type_override=None):
+        """
+        Create a new lock key code.
+
+        Args:
+            device (WinkDevice): The device the change is being requested for.
+            new_device_json (String): The JSON string required to create the device.
+            id_override (String, optional): A device ID used to override the
+                passed in device's ID. Used to make changes on sub-devices.
+                i.e. Outlet in a Powerstrip. The Parent device's ID.
+            type_override (String, optional): Used to override the device type
+                when a device inherits from a device other than WinkDevice.
+        Returns:
+            response_json (Dict): The API's response in dictionary format
+        """
+        object_id = id_override or device.object_id()
+        object_type = type_override or device.object_type()
+        url_string = "{}/{}s/{}/keys".format(self.BASE_URL,
+                                             object_type,
+                                             object_id)
+        try:
+            arequest = requests.post(url_string,
+                                     data=json.dumps(new_device_json),
+                                     headers=API_HEADERS)
+            response_json = arequest.json()
+            return response_json
+        except requests.exceptions.RequestException:
+            return None
 
 
 def disable_local_control():
@@ -259,7 +296,7 @@ def disable_local_control():
 
 
 def set_user_agent(user_agent):
-    _LOGGER.info("Setting user agent to " + user_agent)
+    _LOGGER.info("Setting user agent to %s", user_agent)
     API_HEADERS["User-Agent"] = user_agent
 
 
@@ -272,8 +309,7 @@ def set_bearer_token(token):
 
 
 def legacy_set_wink_credentials(email, password, client_id, client_secret):
-    log_string = "Email: %s Password: %s Client_id: %s Client_secret: %s" % (email, password, client_id, client_secret)
-    _LOGGER.debug(log_string)
+    _LOGGER.debug("Email: %s Password: %s Client_id: %s Client_secret: %s", email, password, client_id, client_secret)
     global CLIENT_ID, CLIENT_SECRET, REFRESH_TOKEN
 
     CLIENT_ID = client_id
@@ -299,9 +335,8 @@ def legacy_set_wink_credentials(email, password, client_id, client_secret):
 
 
 def set_wink_credentials(client_id, client_secret, access_token, refresh_token):
-    log_string = "Client_id: %s Client_secret: %s Access_token: %s Refreash_token: %s" % (client_id, client_secret,
-                                                                                          access_token, refresh_token)
-    _LOGGER.debug(log_string)
+    _LOGGER.debug("Client_id: %s Client_secret: %s Access_token: %s Refreash_token: %s",
+                  client_id, client_secret, access_token, refresh_token)
     global CLIENT_ID, CLIENT_SECRET, REFRESH_TOKEN
 
     CLIENT_ID = client_id
@@ -341,7 +376,7 @@ def refresh_access_token():
 
 
 def get_authorization_url(client_id, redirect_uri):
-    _LOGGER.debug("Client_id: " + client_id + " redirect_uri: " + redirect_uri)
+    _LOGGER.debug("Client_id: %s redirect_uri: %s", client_id, redirect_uri)
     global CLIENT_ID
 
     CLIENT_ID = client_id
@@ -350,7 +385,7 @@ def get_authorization_url(client_id, redirect_uri):
 
 
 def request_token(code, client_secret):
-    _LOGGER.debug("code: " + code + " Client_secret: " + client_secret)
+    _LOGGER.debug("code: %s Client_secret: %s", code, client_secret)
     data = {
         "client_secret": client_secret,
         "grant_type": "authorization_code",
@@ -362,7 +397,7 @@ def request_token(code, client_secret):
     response = requests.post('{}/oauth2/token'.format(WinkApiInterface.BASE_URL),
                              data=json.dumps(data),
                              headers=headers)
-    _LOGGER.debug(response)
+    _LOGGER.debug('%s', response)
     response_json = response.json()
     access_token = response_json.get('access_token')
     refresh_token = response_json.get('refresh_token')
@@ -372,12 +407,12 @@ def request_token(code, client_secret):
 def get_user():
     url_string = "{}/users/me".format(WinkApiInterface.BASE_URL)
     arequest = requests.get(url_string, headers=API_HEADERS)
-    _LOGGER.debug(arequest)
+    _LOGGER.debug('%s', arequest)
     return arequest.json()
 
 
 def get_local_control_access_token(local_control_id):
-    _LOGGER.debug("Local_control_id: " + local_control_id)
+    _LOGGER.debug("Local_control_id: %s", local_control_id)
     if CLIENT_ID and CLIENT_SECRET and REFRESH_TOKEN:
         data = {
             "client_id": CLIENT_ID,
@@ -393,7 +428,7 @@ def get_local_control_access_token(local_control_id):
         response = requests.post('{}/oauth2/token'.format(WinkApiInterface.BASE_URL),
                                  data=json.dumps(data),
                                  headers=headers)
-        _LOGGER.debug(response)
+        _LOGGER.debug('%s', response)
         response_json = response.json()
         access_token = response_json.get('access_token')
         return access_token
@@ -463,9 +498,12 @@ def get_hubs():
     for hub in hubs:
         if hub.manufacturer_device_model() in SUPPORTS_LOCAL_CONTROL:
             _id = hub.local_control_id()
-            token = get_local_control_access_token(_id)
-            ip = hub.ip_address()
-            HUBS[hub.object_id()] = {"ip": ip, "token": token, "id": _id}
+            if _id is not None:
+                token = get_local_control_access_token(_id)
+                ip = hub.ip_address()
+                HUBS[hub.object_id()] = {"ip": ip, "token": token, "id": _id}
+            else:
+                _LOGGER.error("%s is missing local control ID.", hub.name())
     return hubs
 
 
@@ -513,6 +551,10 @@ def get_scenes():
     return get_devices(device_types.SCENE, "scenes")
 
 
+def get_water_heaters():
+    return get_devices(device_types.WATER_HEATER)
+
+
 def get_light_groups():
     light_groups = []
     for group in get_devices(device_types.GROUP, "groups"):
@@ -533,8 +575,11 @@ def get_binary_switch_groups():
 
 def get_subscription_key():
     response_dict = wink_api_fetch()
-    first_device = response_dict.get('data')[0]
-    return get_subscription_key_from_response_dict(first_device)
+    try:
+        first_device = response_dict.get('data')[0]
+        return get_subscription_key_from_response_dict(first_device)
+    except IndexError:
+        raise WinkAPIException("No Wink devices associated with account.")
 
 
 def get_subscription_key_from_response_dict(device):
@@ -546,10 +591,9 @@ def get_subscription_key_from_response_dict(device):
 def wink_api_fetch(end_point='wink_devices'):
     arequest_url = "{}/users/me/{}".format(WinkApiInterface.BASE_URL, end_point)
     response = requests.get(arequest_url, headers=API_HEADERS)
-    _LOGGER.debug(response)
+    _LOGGER.debug('%s', response)
     if response.status_code == 200:
         return response.json()
-
     if response.status_code == 401:
         raise WinkAPIException("401 Response from Wink API.  Maybe Bearer token is expired?")
     else:
@@ -569,6 +613,9 @@ def get_devices(device_type, end_point="wink_devices"):
     elif end_point == "robots" or end_point == "scenes" or end_point == "groups":
         json_data = wink_api_fetch(end_point)
         return get_devices_from_response_dict(json_data, device_type)
+    else:
+        _LOGGER.error("Invalid endpoint %s", end_point)
+        return {}
 
 
 def get_devices_from_response_dict(response_dict, device_type):
